@@ -1,17 +1,15 @@
-# Copyright (C) 2007-2012 Dan Pascu. See LICENSE for details.
-#
 
 """Decorators and helper functions for writing well behaved decorators."""
 
-__all__ = ['decorator', 'preserve_signature', 'execute_once']
-
 from inspect import getargspec, formatargspec
-from itertools import count
 from threading import RLock
 
 from application.python.weakref import weakobjectmap
 
 
+__all__ = ['decorator', 'preserve_signature', 'execute_once']
+
+
 def decorator(func):
     """A syntactic marker with no other effect than improving readability."""
     return func
@@ -19,21 +17,15 @@ def decorator(func):
 
 def preserve_signature(func):
     """Preserve the original function signature and attributes in decorator wrappers."""
-    class ValueFormatter(object):
-        def __init__(self):
-            self.count = count()
-        def __call__(self, value):
-            return "=defaults[%d]" % next(self.count)
-    argspec    = getargspec(func)
-    signature  = formatargspec(*argspec, formatvalue=ValueFormatter())[1:-1]
-    parameters = formatargspec(*argspec, formatvalue=lambda value: "")[1:-1]
     def fix_signature(wrapper):
-        defaults = func.func_defaults
-        code = "def %s(%s): return wrapper(%s)\nnew_wrapper = %s\n" % (func.__name__, signature, parameters, func.__name__)
-        exec code in locals()
+        exec_scope = {}
+        parameters = formatargspec(*getargspec(func), formatvalue=lambda value: "")
+        exec("def {0}{1}: return wrapper{1}".format(func.__name__, parameters), {'wrapper': wrapper}, exec_scope)
+        new_wrapper = exec_scope.pop(func.__name__)
         new_wrapper.__name__ = func.__name__
         new_wrapper.__doc__ = func.__doc__
         new_wrapper.__module__ = func.__module__
+        new_wrapper.__defaults__ = func.__defaults__
         new_wrapper.__dict__.update(func.__dict__)
         return new_wrapper
     return fix_signature
@@ -48,10 +40,12 @@ def execute_once(func):
         pass
 
     class ExecuteOnceMethodWrapper(object):
-        __slots__ = ('__weakref__', '__method__', 'im_func_wrapper', 'called', 'lock')
+        __slots__ = '__weakref__', '__method__', 'im_func_wrapper', 'called', 'lock'
+
         def __init__(self, method, func_wrapper):
             self.__method__ = method
             self.im_func_wrapper = func_wrapper
+
         def __call__(self, *args, **kw):
             with self.im_func_wrapper.lock:
                 method = self.__method__
@@ -62,39 +56,49 @@ def execute_once(func):
                 self.im_func_wrapper.__callmap__[instance] = True
                 self.im_func_wrapper.__callmap__[method.im_class] = True
                 return method.__call__(*args, **kw)
+
         def __dir__(self):
             return sorted(set(dir(self.__method__) + dir(self.__class__) + list(self.__slots__)))
+
         def __get__(self, obj, cls):
             method = self.__method__.__get__(obj, cls)
             return self.__class__(method, self.im_func_wrapper)
+
         def __getattr__(self, name):
             return getattr(self.__method__, name)
+
         def __setattr__(self, name, value):
             if name in self.__slots__:
                 object.__setattr__(self, name, value)
             else:
                 setattr(self.__method__, name, value)
+
         def __delattr__(self, name):
             if name in self.__slots__:
                 object.__delattr__(self, name)
             else:
                 delattr(self.__method__, name)
+
         def __repr__(self):
             return self.__method__.__repr__().replace('<', '<wrapper of ', 1)
+
         @property
         def called(self):
             return self.im_func_wrapper.__callmap__.get(self.__method__.im_self or self.__method__.im_class, False)
+
         @property
         def lock(self):
             return self.im_func_wrapper.lock
 
     class ExecuteOnceFunctionWrapper(object):
-        __slots__ = ('__weakref__', '__func__', '__callmap__', 'called', 'lock')
+        __slots__ = '__weakref__', '__func__', '__callmap__', 'called', 'lock'
+
         def __init__(self, func):
             self.__func__ = func
             self.__callmap__ = weakobjectmap()
             self.__callmap__[func] = False
             self.lock = RLock()
+
         def __call__(self, *args, **kw):
             with self.lock:
                 check_arguments(*args, **kw)
@@ -102,25 +106,32 @@ def execute_once(func):
                     return
                 self.__callmap__[self.__func__] = True
                 return self.__func__.__call__(*args, **kw)
+
         def __dir__(self):
             return sorted(set(dir(self.__func__) + dir(self.__class__) + list(self.__slots__)))
+
         def __get__(self, obj, cls):
             method = self.__func__.__get__(obj, cls)
             return ExecuteOnceMethodWrapper(method, self)
+
         def __getattr__(self, name):
             return getattr(self.__func__, name)
+
         def __setattr__(self, name, value):
             if name in self.__slots__:
                 object.__setattr__(self, name, value)
             else:
                 setattr(self.__func__, name, value)
+
         def __delattr__(self, name):
             if name in self.__slots__:
                 object.__delattr__(self, name)
             else:
                 delattr(self.__func__, name)
+
         def __repr__(self):
             return self.__func__.__repr__().replace('<', '<wrapper of ', 1)
+
         @property
         def called(self):
             return self.__callmap__[self.__func__]
